/*
 * Copyright (C) 2003 Hans Karlsson <karlsson.h@home.se>
 */

#include "karamba.h"

// Menu IDs
#define EDITSCRIPT 1

/* now a method we need to expose to Python */
long toggleShowDesktop(long widget) {
  karamba* currTheme = (karamba*)widget;
  
  ShowDesktop *s = ShowDesktop::the();

  s->toggle();

  return 1;
}

static PyObject* py_toggle_show_desktop(PyObject *self, PyObject *args)
{
  long widget, image, deg;
  if (!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return Py_BuildValue("l", toggleShowDesktop(widget));
}


// This does something with a task, such as minimize or close it
int performTaskAction(long widget, long ctask, long action) {
  karamba* currTheme = (karamba*)widget;
  Task* currTask = 0;
  Task *task;


  for ( task = currTheme->taskList.first(); task; task = currTheme->taskList.next() ) {
    
    if ((int)task == (int)ctask) {
      currTask = task;
    }
    
  }
  
  if (currTask != 0) {

    switch (action) {
    case 1:
      currTask->maximize();
      break;

    case 2:
      currTask->restore();
      break;

    case 3:
      currTask->iconify();
      break;

    case 4:
      currTask->close();
      break;

    case 5:
      currTask->activate();
      break;

    case 6:
      currTask->raise();
      break;

    case 7:
      currTask->lower();
      break;

    case 8:
      currTask->activateRaiseOrIconify();
      break;

    case 9:
      currTask->toggleAlwaysOnTop();
      break;

    case 10:
      currTask->toggleShaded();
      break;

    default:
      printf("You are trying to perform an invalid action in performTaskAction\n");
    }
    

    return 0;
    
  } else {
    return 1;
  }

}

static PyObject* py_perform_task_action(PyObject *self, PyObject *args)
{
  long widget, task, action;
  if (!PyArg_ParseTuple(args, "lll", &widget, &task, &action))
    return NULL;
  return Py_BuildValue("l", performTaskAction(widget, task, action));
}


// This returns all the info about a certain task
// Return type is a Python List
PyObject* getTaskInfo(long widget, long ctask) {
  karamba* currTheme = (karamba*)widget;
  Task* currTask = 0;
  Task *task;

  for ( task = currTheme->taskList.first(); task; task = currTheme->taskList.next() ) {
    
    if ((int)task == (int)ctask) {
      currTask = task;
    }
    
  }
  
  if (currTask != 0) {


    PyObject* pList = PyList_New(0);
    PyObject* pString;
    PyObject* pInt;
    
    //Task Name
    if (currTask->name() != NULL) {
      pString = PyString_FromString(currTask->name().latin1());
    } else {
      pString = PyString_FromString("");      
    }
    PyList_Append(pList, pString);
    
    //Icon Name
    if (currTask->iconName() != NULL) {
      pString = PyString_FromString(currTask->iconName().latin1());
    } else {
      pString = PyString_FromString("");      
    }
    PyList_Append(pList, pString);

    //Class Name
    if (currTask->className() != NULL) {
      pString = PyString_FromString(currTask->className().latin1());
    } else {
      pString = PyString_FromString("");      
    }
    PyList_Append(pList, pString);
    
    // Desktop this task is on
    pInt = PyInt_FromLong(currTask->desktop());
    PyList_Append(pList, pInt);
    
    // is it maximized?
    pInt = PyInt_FromLong(currTask->isMaximized());
    PyList_Append(pList, pInt);

    // is it iconified?
    pInt = PyInt_FromLong(currTask->isIconified());
    PyList_Append(pList, pInt);

    // is it shaded?
    pInt = PyInt_FromLong(currTask->isShaded());
    PyList_Append(pList, pInt);

    // is it focused?
    pInt = PyInt_FromLong(currTask->isActive());
    PyList_Append(pList, pInt);

    return pList;

  } else {
    return NULL;
  }

}

static PyObject* py_get_task_info(PyObject *self, PyObject *args)
{
  long widget, task;
  if (!PyArg_ParseTuple(args, "ll", &widget, &task))
    return NULL;
  return getTaskInfo(widget, task);
}



// This gets a system task list
// It returns a String List of task names
PyObject* getTaskNames(long widget) {
  karamba* currTheme = (karamba*)widget;
  PyObject* pList = PyList_New(0);
  PyObject* pString;

  currTheme->taskList = currTheme->taskManager.tasks();

    Task *task;
    for ( task = currTheme->taskList.first(); task; task = currTheme->taskList.next() ) {
	
	pString = PyString_FromString(task->name().latin1());
	PyList_Append(pList, pString);
	
    }
    

    return pList;
}

static PyObject* py_get_task_names(PyObject *self, PyObject *args)
{
  long widget;
  if(!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return getTaskNames(widget);
}

// This gets a system task list
PyObject* getTaskList(long widget) {
  karamba* currTheme = (karamba*)widget;
  PyObject* pList = PyList_New(0);
  PyObject* pString;

  currTheme->taskList = currTheme->taskManager.tasks();

    Task *task;
    for ( task = currTheme->taskList.first(); task; task = currTheme->taskList.next() ) {
	
	pString = PyInt_FromLong((int)task);
	PyList_Append(pList, pString);
	
    }
    

    return pList;
}

static PyObject* py_get_task_list(PyObject *self, PyObject *args)
{
  long widget;
  if(!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return getTaskList(widget);
}

/* now a method we need to expose to Python */
int getWidgetXCoordinate(long widget) {
  karamba* currTheme = (karamba*)widget;
  
  return currTheme->x();
}

/* now a method we need to expose to Python */
int getWidgetYCoordinate(long widget) {
  karamba* currTheme = (karamba*)widget;
  
  return currTheme->y();
}

static PyObject* py_get_widget_position(PyObject *self, PyObject *args)
{
  long widget;
  if(!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return Py_BuildValue("(i,i)", getWidgetXCoordinate(widget), getWidgetYCoordinate(widget));
}


/* now a method we need to expose to Python */
const char* getThemePath(long widget) {
  karamba* currTheme = (karamba*)widget;

  return currTheme->themePath.ascii();
}

static PyObject* py_get_theme_path(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return Py_BuildValue("s", getThemePath(widget));
}


/* now a method we need to expose to Python */
long createWidgetMask(long widget, char* path) {
  karamba* currTheme = (karamba*)widget;

  currTheme->clearMask();

  if (currTheme->widgetMask != 0) {
    delete currTheme->widgetMask;
  }

  QString maskpath;
  maskpath.setAscii(path);

  currTheme->widgetMask = new QBitmap(path);

  currTheme->kroot->stop();
  delete currTheme->kroot;

  //  currTheme->setMask(*(currTheme->widgetMask));

  QBitmap blah = QBitmap("/home/ageitgey/mask.png");
  currTheme->setMask(blah);


  currTheme->kroot = new KarambaRootPixmap((QWidget*)currTheme);
  currTheme->kroot->start();

  return (int)currTheme->widgetMask;
}

static PyObject* py_create_widget_mask(PyObject *self, PyObject *args)
{
  long widget;
  char *text;
  if (!PyArg_ParseTuple(args, "ls", &widget, &text))
    return NULL;
  return Py_BuildValue("l", createWidgetMask(widget, text));
}

long createMenu(long widget){
  karamba* currTheme = (karamba*)widget;

  KPopupMenu* tmp = new KPopupMenu(currTheme);
  currTheme->menuList->append (tmp );

  currTheme->connect(tmp, SIGNAL(activated(int)), currTheme, SLOT(passMenuItemClicked(int)));

  return (int)tmp;
}

static PyObject* py_create_menu(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return Py_BuildValue("l", createMenu(widget));
}

bool menuExists(karamba* currTheme, KPopupMenu* menu){
  bool foundMenu = false;
  KPopupMenu* tmp;

  for(int i = 0; i < currTheme->menuList->count(); i++){
    if(i==0){
      tmp = (KPopupMenu*) currTheme->menuList->first();
    } else {
      tmp = (KPopupMenu*) currTheme->menuList->next();
    }
    if(tmp != 0){
      if(tmp == menu){
        foundMenu = true;
        break;
      }
    }
  }
  return foundMenu;
}


long deleteMenu(long widget, long menu){
  karamba* currTheme = (karamba*)widget;
  KPopupMenu* tmp = (KPopupMenu*)menu;

  currTheme->menuList->removeRef(tmp);

  return 1;
}

static PyObject* py_delete_menu(PyObject *self, PyObject *args)
{
  long widget, menu;
  if (!PyArg_ParseTuple(args, "ll", &widget, &menu))
    return NULL;
  return Py_BuildValue("l", deleteMenu(widget, menu));
}

long addMenuItem(long widget, long menu, char* text, char* icon){
  karamba* currTheme = (karamba*)widget;
  KPopupMenu* tmp = (KPopupMenu*)menu;

  long id = 0;
  if(menuExists(currTheme, tmp)){
    QString label;
    label.setAscii(text);
    QString smallicon;
    smallicon.setAscii(icon);

    id = tmp->insertItem(SmallIconSet(smallicon), label);
  }

  return id;
}

static PyObject* py_add_menu_item(PyObject *self, PyObject *args)
{
  long widget, menu;
  char* icon;
  char* text;
  if (!PyArg_ParseTuple(args, "llss", &widget, &menu, &text, &icon))
    return NULL;
  return Py_BuildValue("l", addMenuItem(widget, menu, text, icon));
}

long removeMenuItem(long widget, long menu, long id){
  karamba* currTheme = (karamba*)widget;
  KPopupMenu* tmp = (KPopupMenu*)menu;

  if(menuExists(currTheme,tmp)){
    tmp->removeItem(id);
    return 1;
  } else {
    return 0;
  }
}

static PyObject* py_remove_menu_item(PyObject *self, PyObject *args)
{
  long widget, menu, id;
  if (!PyArg_ParseTuple(args, "lll", &widget, &menu, &id))
    return NULL;
  return Py_BuildValue("l", removeMenuItem(widget, menu, id));
}

long popupMenu(long widget, long menu, long x, long y){
  karamba* currTheme = (karamba*)widget;
  KPopupMenu* tmp = (KPopupMenu*)menu;

  if(menuExists(currTheme,tmp)){
    tmp->popup(QPoint(x,y));
    return 1;
  } else {
    return 0;
  }
}

static PyObject* py_popup_menu(PyObject *self, PyObject *args)
{
  long widget, menu, x, y;
  if (!PyArg_ParseTuple(args, "llll", &widget, &menu, &x, &y))
    return NULL;
  return Py_BuildValue("l", popupMenu(widget, menu, x, y));
}


// /* now a method we need to expose to Python */
// long createPopupItem(long widget, char* icon, char* text, long ident) {
//   int menuIndex = 0;
//   karamba* currTheme = (karamba*)widget;

//   QString smallicon;
//   smallicon.setAscii(icon);

//   QString value;
//   value.setAscii(text);

//   menuIndex = currTheme->kpop->insertItem( SmallIconSet(smallicon), value, currTheme, SLOT(popupNotify()));

//   //currTheme->kpop->setItemParameter(menuIndex, (int)ident);

//   return 1;
// }

// static PyObject* py_create_popup_item(PyObject *self, PyObject *args)
// {
//   long widget, ident;
//   char* icon, text;
//   if (!PyArg_ParseTuple(args, "lssl", &widget, &icon, &text, &ident))
//     return NULL;
//   return Py_BuildValue("l", createPopupItem(widget, icon, text, ident));
// }



/* now a method we need to expose to Python */
long createClickArea(long widget, long x, long y, long w, long h, char* text) {

  karamba* currTheme = (karamba*)widget;
  ClickArea *tmp = new ClickArea( x, y, w, h );
  QString onclick;
  
  onclick.setAscii(text);

  tmp->setOnClick(onclick );

  currTheme->clickList->push_back( tmp );
  return (int)tmp;
}

static PyObject* py_create_click_area(PyObject *self, PyObject *args)
{
  long widget, x, y, w, h;
  char *text;
  if (!PyArg_ParseTuple(args, "llllls", &widget, &x, &y, &w, &h, &text))
    return NULL;
  return Py_BuildValue("l", createClickArea(widget, x, y, w, h, text));
}


/* now a method we need to expose to Python */
long removeImageEffects(long widget, long image) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->removeEffects();

  return 1;
}

static PyObject* py_remove_image_effects(PyObject *self, PyObject *args)
{
  long widget, image;
  if (!PyArg_ParseTuple(args, "ll", &widget, &image))
    return NULL;
  return Py_BuildValue("l", removeImageEffects(widget, image));
}

/* now a method we need to expose to Python */
long changeImageIntensity(long widget, long image, float ratio, int millisec) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->intensity(ratio, millisec);

  return 1;
}

static PyObject* py_change_image_intensity(PyObject *self, PyObject *args)
{
  long widget, image, millisec;
	float ratio;
  if (!PyArg_ParseTuple(args, "llfl", &widget, &image, &ratio, &millisec))
    return NULL;
  return Py_BuildValue("l", changeImageIntensity(widget, image, ratio, millisec));
}

/* now a method we need to expose to Python */
long changeImageChannelIntensity(long widget, long image, float ratio, char* channel, int millisec) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

	QString qChannel;
	qChannel.setAscii(channel);

  currSensor->channelIntensity(ratio, qChannel, millisec);

  return 1;
}

static PyObject* py_change_image_channel_intensity(PyObject *self, PyObject *args)
{
  long widget, image, millisec;
	float ratio;
	char* channel;
  if (!PyArg_ParseTuple(args, "llfsl", &widget, &image, &ratio, &channel, &millisec))
    return NULL;
  return Py_BuildValue("l", changeImageChannelIntensity(widget, image, ratio, channel, millisec));
}

/* now a method we need to expose to Python */
long changeImageToGray(long widget, long image, int millisec) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->toGray(millisec);

  return 1;
}

static PyObject* py_change_image_toGray(PyObject *self, PyObject *args)
{
  long widget, image, millisec;
  if (!PyArg_ParseTuple(args, "lll", &widget, &image, &millisec))
    return NULL;
  return Py_BuildValue("l", changeImageToGray(widget, image, millisec));
}




long removeImageTransformations(long widget, long image) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->removeImageTransformations();

  return 1;
}

static PyObject* py_remove_image_transformations(PyObject *self, PyObject *args)
{
  long widget, image;
  if (!PyArg_ParseTuple(args, "ll", &widget, &image))
    return NULL;
  return Py_BuildValue("l", removeImageTransformations(widget, image));
}


/* now a method we need to expose to Python */
long rotateImage(long widget, long image, long deg) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->rotate((int)deg);

  return 1;
}

static PyObject* py_rotate_image(PyObject *self, PyObject *args)
{
  long widget, image, deg;
  if (!PyArg_ParseTuple(args, "lll", &widget, &image, &deg))
    return NULL;
  return Py_BuildValue("l", rotateImage(widget, image, deg));
}

/* now a method we need to expose to Python */
long getImageHeight(long widget, long image) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  return currSensor->getHeight();
}

static PyObject* py_get_image_height(PyObject *self, PyObject *args)
{
  long widget, image;
  if (!PyArg_ParseTuple(args, "ll", &widget, &image))
    return NULL;
  return Py_BuildValue("l", getImageHeight(widget, image));
}

/* now a method we need to expose to Python */
long getImageWidth(long widget, long image) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  return currSensor->getWidth();
}

static PyObject* py_get_image_width(PyObject *self, PyObject *args)
{
  long widget, image;
  if (!PyArg_ParseTuple(args, "ll", &widget, &image))
    return NULL;
  return Py_BuildValue("l", getImageWidth(widget, image));
}

/* now a method we need to expose to Python */
long resizeImageSmooth(long widget, long image, long w, long h) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->smoothScale((int)w, (int)h);

  return 1;
}

static PyObject* py_resize_image_smooth(PyObject *self, PyObject *args)
{
  long widget, image, w, h;
  if (!PyArg_ParseTuple(args, "llll", &widget, &image, &w, &h))
    return NULL;
  return Py_BuildValue("l", resizeImageSmooth(widget, image, w, h));
}


/* now a method we need to expose to Python */
long resizeImage(long widget, long image, long w, long h) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->scale((int)w, (int)h);

  return 1;
}

static PyObject* py_resize_image(PyObject *self, PyObject *args)
{
  long widget, image, w, h;
  if (!PyArg_ParseTuple(args, "llll", &widget, &image, &w, &h))
    return NULL;
  return Py_BuildValue("l", resizeImage(widget, image, w, h));
}


/* now a method we need to expose to Python */
long showImage(long widget, long image) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->showImage();

  return 1;
}

static PyObject* py_show_image(PyObject *self, PyObject *args)
{
  long widget, image;
  if (!PyArg_ParseTuple(args, "ll", &widget, &image))
    return NULL;
  return Py_BuildValue("l", showImage(widget, image));
}

/* now a method we need to expose to Python */
long hideImage(long widget, long image) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->hideImage();

  return 1;
}

static PyObject* py_hide_image(PyObject *self, PyObject *args)
{
  long widget, image;
  if (!PyArg_ParseTuple(args, "ll", &widget, &image))
    return NULL;
  return Py_BuildValue("l", hideImage(widget, image));
}

/* now a method we need to expose to Python */
long moveImage(long widget, long image, long x, long y) {
  karamba* currTheme = (karamba*)widget;
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->setSize((int)x, (int)y, 0, 0);

  return 1;
}

static PyObject* py_move_image(PyObject *self, PyObject *args)
{
  long widget, image, x, y;
  if (!PyArg_ParseTuple(args, "llll", &widget, &image, &x, &y))
    return NULL;
  return Py_BuildValue("l", moveImage(widget, image, x, y));
}

/* now a method we need to expose to Python */
long addImageTooltip(long widget, long image, char* chartiptext) {
  karamba* currTheme = (karamba*)widget;
  
  QString tiptext;
  tiptext.setAscii(chartiptext);
  
  ImageLabel* currSensor = (ImageLabel*)image;

  currSensor->setTooltip(tiptext);
  
  return (int)1;
}

static PyObject* py_add_image_tooltip(PyObject *self, PyObject *args)
{
  long widget, image;
  char *text;
  if (!PyArg_ParseTuple(args, "lls", &widget, &image, &text))
    return NULL;
  return Py_BuildValue("l", addImageTooltip(widget, image, text));
}


/* now a method we need to expose to Python */
long deleteImage(long widget, long image) {
  karamba* currTheme = (karamba*)widget;
  int res1, res2;

  ImageLabel *tmp = (ImageLabel*)image;

  res2 = currTheme->imageList->removeRef(tmp);
  res1 = currTheme->meterList->removeRef(tmp);

  //meterList is autoDelete(true) so we don't have to delete the image here
    
  return 1;
}

static PyObject* py_delete_image(PyObject *self, PyObject *args)
{
  long widget, image;

  if (!PyArg_ParseTuple(args, "ll", &widget, &image))
    return NULL;
  return Py_BuildValue("l", deleteImage(widget, image));
}

/* now a method we need to expose to Python */
long createImage(long widget, long x, long y, char* path) {
  karamba* currTheme = (karamba*)widget;
  
  QString file;
  file.setAscii(path);
  
  QString fakefile;

  ImageLabel *tmp = new ImageLabel(x,y,0,0);
  tmp->setThemePath( currTheme->themePath );
  tmp->setValue( file );
  tmp->parseImages(file, fakefile, x,y, 0, 0);
  tmp->setKaramba(currTheme);
    
  currTheme->setSensor(file, (Meter*) tmp );
  currTheme->meterList->append (tmp );
  currTheme->imageList->append (tmp );
    
  return (int)tmp;
}

static PyObject* py_create_image(PyObject *self, PyObject *args)
{
  long widget, x, y;
  char *text;
  if (!PyArg_ParseTuple(args, "llls", &widget, &x, &y, &text))
    return NULL;
  return Py_BuildValue("l", createImage(widget, x, y, text));
}



/* now a method we need to expose to Python */
long showText(long widget, long text) {
  karamba* currTheme = (karamba*)widget;
  TextLabel* currSensor = (TextLabel*)text;

  currSensor->showText();

  return 1;
}

static PyObject* py_show_text(PyObject *self, PyObject *args)
{
  long widget, text;
  if (!PyArg_ParseTuple(args, "ll", &widget, &text))
    return NULL;
  return Py_BuildValue("l", showText(widget, text));
}

/* now a method we need to expose to Python */
long hideText(long widget, long text) {
  karamba* currTheme = (karamba*)widget;
  TextLabel* currSensor = (TextLabel*)text;

  currSensor->hideText();

  return 1;
}

static PyObject* py_hide_text(PyObject *self, PyObject *args)
{
  long widget, text;
  if (!PyArg_ParseTuple(args, "ll", &widget, &text))
    return NULL;
  return Py_BuildValue("l", hideText(widget, text));
}


/* now a method we need to expose to Python */
long moveText(long widget, long text, long x, long y) {
  karamba* currTheme = (karamba*)widget;
  TextLabel* currSensor = (TextLabel*)text;

  currSensor->setX((int)x);
  currSensor->setY((int)y);

  return 1;
}

static PyObject* py_move_text(PyObject *self, PyObject *args)
{
  long widget, text, x, y;
  if (!PyArg_ParseTuple(args, "llll", &widget, &text, &x, &y))
    return NULL;
  return Py_BuildValue("l", moveText(widget, text, x, y));
}


/* now a method we need to expose to Python */
long changeTextShadow(long widget, long textSensor, long size) {
  TextLabel* currSensor = (TextLabel*)textSensor;
  karamba* currTheme = (karamba*)widget;

  currSensor->setShadow( size );


  return 1;
}

static PyObject* py_change_text_shadow(PyObject *self, PyObject *args)
{
  long widget, textSensor;
  long shadow;
  if (!PyArg_ParseTuple(args, "lll", &widget, &textSensor, &shadow))
    return NULL;
  return Py_BuildValue("l", changeTextShadow(widget, textSensor, shadow));
}


/* now a method we need to expose to Python */
long changeTextSize(long widget, long textSensor, long size) {
  TextLabel* currSensor = (TextLabel*)textSensor;
  karamba* currTheme = (karamba*)widget;

  currSensor->setFontSize( size );


  return 1;
}

static PyObject* py_change_text_size(PyObject *self, PyObject *args)
{
  long widget, textSensor;
  long size;
  if (!PyArg_ParseTuple(args, "lll", &widget, &textSensor, &size))
    return NULL;
  return Py_BuildValue("l", changeTextSize(widget, textSensor, size));
}


/* now a method we need to expose to Python */
long changeTextColor(long widget, long textSensor, long r, long g, long b) {
  TextLabel* currSensor = (TextLabel*)textSensor;
  karamba* currTheme = (karamba*)widget;

  currSensor->setColor( r, g, b );

  return 1;
}

static PyObject* py_change_text_color(PyObject *self, PyObject *args)
{
  long widget, textSensor;
  long r, g, b;
  if (!PyArg_ParseTuple(args, "lllll", &widget, &textSensor, &r, &g, &b))
    return NULL;
  return Py_BuildValue("l", changeTextColor(widget, textSensor, r, g, b));
}


/* now a method we need to expose to Python */
long changeTextFont(long widget, long textSensor, char* newFont) {
  TextLabel* currSensor = (TextLabel*)textSensor;
  karamba* currTheme = (karamba*)widget;

  QString value;
  value.setAscii(newFont);
  currSensor->setFont( value );

  return 1;
}

static PyObject* py_change_text_font(PyObject *self, PyObject *args)
{
  long widget, textSensor;
  char* text;
  if (!PyArg_ParseTuple(args, "lls", &widget, &textSensor, &text))
    return NULL;
  return Py_BuildValue("l", changeTextFont(widget, textSensor, text));
}


/* now a method we need to expose to Python */
long changeText(long widget, long textSensor, char* newText) {
  TextLabel* currSensor = (TextLabel*)textSensor;
  karamba* currTheme = (karamba*)widget;

  QString value;
  value.setAscii(newText);
  currSensor->setValue(value);

  return 1;
}

static PyObject* py_change_text(PyObject *self, PyObject *args)
{
  long widget, textSensor;
  char* text;
  if (!PyArg_ParseTuple(args, "lls", &widget, &textSensor, &text))
    return NULL;
  return Py_BuildValue("l", changeText(widget, textSensor, text));
}

/* now a method we need to expose to Python */
long deleteText(long widget, long text) {
  karamba* currTheme = (karamba*)widget;
  int res1;

  TextLabel *tmp = (TextLabel*)text;

  res1 = currTheme->meterList->removeRef(tmp);

  //meterList is autoDelete(true) so we don't have to delete the image here
    
  return 1;
}

static PyObject* py_delete_text(PyObject *self, PyObject *args)
{
  long widget, text;
 
  if (!PyArg_ParseTuple(args, "ll", &widget, &text))
    return NULL;
  return Py_BuildValue("l", deleteText(widget, text));
}


/* now a method we need to expose to Python */
long createText(long widget, long x, long y, long w, long h, char* text) {
  karamba* currTheme = (karamba*)widget;


  TextLabel *tmp = new TextLabel((int)x, (int)y, (int)w, (int)h );
//   bool paramFound = false;
  
//   get3Int( "COLOR", line, r, g, b, paramFound );
//   if ( paramFound || !defaultTextLabel )
//     tmp->setColor ( r, g, b );
//   else
//     if ( defaultTextLabel )
//       {
// 	defaultTextLabel->getColor().rgb ( &r, &g, &b );
  tmp->setColor( 128, 128, 128 );
//       }
  
//   get3Int( "BGCOLOR", line, r, g, b, paramFound );
//   if ( paramFound  || !defaultTextLabel )
//     tmp->setBGColor( r, g, b );
//   else
//     if ( defaultTextLabel )
//       {
// 	defaultTextLabel->getBGColor().rgb ( &r, &g, &b );
// 	tmp->setBGColor( r, g, b );
//       }
  
//   QString dFont = getString("FONT", line, paramFound);
//   if( paramFound || !defaultTextLabel )
//     tmp->setFont( dFont );
//   else
//     if( defaultTextLabel )
//       tmp->setFont( defaultTextLabel->getFont() );
  
  
//   int dFontSize = getInt( "FONTSIZE", line, paramFound );
//   if( paramFound || !defaultTextLabel )
//     tmp->setFontSize( dFontSize );
//   else
//     if( defaultTextLabel )
//       tmp->setFontSize( defaultTextLabel->getFontSize() );
  
//   QString dAlign = getString( "ALIGN", line, paramFound );
//   if( paramFound || !defaultTextLabel )
                    tmp->setAlignment( "LEFT" );
//   else
//     if( defaultTextLabel )
//       tmp->setAlignment( defaultTextLabel->getAlignment() );
  
//   bool dFp = getBoolean( "FIXEDPITCH", line, paramFound );
//   if( paramFound || !defaultTextLabel )
//     tmp->setFixedPitch( dFp );
//   else
//     if( defaultTextLabel )
//       tmp->setFixedPitch( defaultTextLabel->getFixedPitch() );
  
//   int dShadow = getInt( "SHADOW", line, paramFound );
//   if( paramFound || !defaultTextLabel )
     tmp->setShadow( 0 );
//   else
//     if( defaultTextLabel )
//       tmp->setShadow( defaultTextLabel->getShadow() );
  
//  printf("%s\n", text);
  QString line;
  QString value;
  line.setAscii(text);
  value.setAscii(text);
  tmp->setValue(value);
  //  currTheme->setSensor(line, (Meter*)tmp );
  currTheme->meterList->append ( tmp );

  return (int)tmp;
}

static PyObject* py_create_text(PyObject *self, PyObject *args)
{
  long widget, x, y, w, h;
  char *text;
  if (!PyArg_ParseTuple(args, "llllls", &widget, &x, &y, &w, &h, &text))
    return NULL;
  return Py_BuildValue("l", createText(widget, x, y, w, h, text));
}

/* now a method we need to expose to Python */
long redrawWidget(long widget) {
  karamba* currTheme = (karamba*)widget;
  currTheme->externalStep();
  return 1;
}

static PyObject* py_redraw_widget(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l", &widget))
    return NULL;
  return Py_BuildValue("l", redrawWidget(widget));
}


/* now a method we need to expose to Python */
long resizeWidget(long widget, long x, long y) {
  karamba* currTheme = (karamba*)widget;
  currTheme->setFixedSize((int)x,(int)y);
  return 1;
}

static PyObject* py_resize_widget(PyObject *self, PyObject *args)
{
  long widget, x, y;
  if (!PyArg_ParseTuple(args, "lll", &widget, &x, &y))
    return NULL;
  return Py_BuildValue("l", resizeWidget(widget, x, y));
}

/* now a method we need to expose to Python */
long moveWidget(long widget, long x, long y) {
  karamba* currTheme = (karamba*)widget;
  currTheme->move((int)x, (int)y);
  return 1;
}

static PyObject* py_move_widget(PyObject *self, PyObject *args)
{
  long widget, x, y;
  if (!PyArg_ParseTuple(args, "lll", &widget, &x, &y))
    return NULL;
  return Py_BuildValue("l", moveWidget(widget, x, y));
}

static PyMethodDef karamba_methods[] = {
  {"moveWidget",     py_move_widget,     METH_VARARGS,  "Move Widget to x,y"},
  {"resizeWidget",     py_resize_widget,     METH_VARARGS,  "Resize Widget to width,height"},
  {"createWidgetMask",     py_create_widget_mask,     METH_VARARGS,  "Create a clipping mask for this widget"},
  {"redrawWidget",     py_redraw_widget,     METH_VARARGS,  "Update Widget to reflect your changes"},
  {"getThemePath",     py_get_theme_path,     METH_VARARGS,  "Get the file path of the theme"},
  {"toggleShowDesktop",     py_toggle_show_desktop,     METH_VARARGS,  "Show/Hide the desktop"},
  {"createClickArea",     py_create_click_area,     METH_VARARGS,  "Create a Click Area Sensor"},
  {"getWidgetPosition", py_get_widget_position, METH_VARARGS, "Get Widget Position"},
  {"createMenu",        py_create_menu, METH_VARARGS, "Create a popup menu"},
  {"deleteMenu",        py_delete_menu, METH_VARARGS, "Delete a popup menu"},
  {"addMenuItem",       py_add_menu_item, METH_VARARGS, "Add a popup menu entry"},
  {"removeMenuItem",    py_remove_menu_item, METH_VARARGS, "Remove a popup menu entry"},
  {"popupMenu",         py_popup_menu, METH_VARARGS, "Popup a menu at a specified location"},
  //  {"createPopupItem",     py_create_popup_item,     METH_VARARGS,  "Create an item in the Widget's Popup Menu"},
  {"removeImageTransformations", py_remove_image_transformations, METH_VARARGS, "Restore original size and orientation of an Image"},
  {"removeImageEffects", py_remove_image_effects, METH_VARARGS, "Remove Effects of an Image"},
  {"changeImageIntensity", py_change_image_intensity, METH_VARARGS, "Change Intensity of an Image"},
  {"changeImageChannelIntensity", py_change_image_channel_intensity, METH_VARARGS, "Change Intensity of an Image Channel"},
  {"changeImageToGray", py_change_image_toGray, METH_VARARGS, "Converts an Image to Grayscale"},

  {"createText",     py_create_text,     METH_VARARGS,  "Create a Text Sensor"},
  {"deleteText",     py_delete_text,     METH_VARARGS,  "Delete a Text object"},
  {"moveText",     py_move_text,     METH_VARARGS,  "Move a Text object"},
  {"changeText",     py_change_text,     METH_VARARGS,  "Change a Text Sensor's Text"},
  {"hideText",     py_hide_text,     METH_VARARGS,  "Hide a Text object"},
  {"showText",     py_show_text,     METH_VARARGS,  "Show a Text object"},
  {"changeTextShadow",     py_change_text_shadow,     METH_VARARGS,  "Change a Text Shadow size"},
  {"changeTextFont",     py_change_text_font,     METH_VARARGS,  "Change a Text Sensor's Font"},
  {"changeTextColor",     py_change_text_color,     METH_VARARGS,  "Change a Text Sensor's Color"},
  {"changeTextSize",     py_change_text_size,     METH_VARARGS,  "Change a Text Sensor's Font Size"},
  {"createImage",     py_create_image,     METH_VARARGS,  "Create an Image"},
  {"deleteImage",     py_delete_image,     METH_VARARGS,  "Delete an Image"},
  {"addImageTooltip",     py_add_image_tooltip,     METH_VARARGS,  "Create a Tooltip for an Image"},
  {"moveImage",     py_move_image,     METH_VARARGS,  "Move an Image"},
  {"hideImage",     py_hide_image,     METH_VARARGS,  "Hide an Image"},
  {"showImage",     py_show_image,     METH_VARARGS,  "Show an Image"},
  {"resizeImage",     py_resize_image,     METH_VARARGS,  "Scale an Image"},
  {"resizeImageSmooth",     py_resize_image_smooth,     METH_VARARGS,  "Scale an Image (slower, better looking)"},
  {"rotateImage",     py_rotate_image,     METH_VARARGS,  "Rotate an Image"},
  {"getImageWidth",     py_get_image_width,     METH_VARARGS,  "Get the width of an Image"},
  {"getImageHeight",     py_get_image_height,     METH_VARARGS,  "Get the height of an Image"},
  {"getTaskList",     py_get_task_list,     METH_VARARGS,  "Get the system task list"},
  {"getTaskNames",     py_get_task_names,     METH_VARARGS,  "Get the system task list in name form"},
  {"getTaskInfo",     py_get_task_info,     METH_VARARGS,  "Get all the info for a task"},
  {"performTaskAction",     py_perform_task_action,     METH_VARARGS,  "Do something with a task, such as minimize it"},
  {NULL, NULL, 0 ,NULL}
   };



karamba::karamba(QString fn, bool reloading)
        :  QWidget( 0,"karamba", Qt::WGroupLeader | WStyle_Customize |  WRepaintNoErase| WStyle_NoBorder | WDestructiveClose  )
{

    char pypath[1024];

    themePath = QFileInfo( fn ).dirPath(true);
    themeName = QFileInfo( fn ).baseName();
   

    connect(qApp,SIGNAL(lastWindowClosed()),qApp,SLOT(quit()));

    pythonThemeExtensionLoaded = false;

    //Add theme path to python path so that we can find the python file
    snprintf(pypath, 1023, "sys.path.insert(0, '%s')", themePath.ascii()); 

    // get the global python thread lock
    PyEval_AcquireLock();

    // get a reference to the PyInterpreterState
    extern PyThreadState * mainThreadState;
    PyInterpreterState * mainInterpreterState = mainThreadState->interp;

    // create a thread state object for this thread
    PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
    PyThreadState_Swap(myThreadState);
    
    // load the .py file for this .theme
    PyRun_SimpleString("import sys");
    PyRun_SimpleString(pypath);
    PyRun_SimpleString("sys.path.insert(0, '')");

        
    PyImport_AddModule("karamba");
    Py_InitModule("karamba", karamba_methods);


    pName = PyString_FromString(themeName.ascii());

    pModule = PyImport_Import(pName);

    //Make sure the module is up to date.
    if (reloading)
      PyImport_ReloadModule(pModule);

    if (pModule != NULL) {
	pythonThemeExtensionLoaded = true;
	pDict = PyModule_GetDict(pModule);

	//HERE
    }
     else {
       PyErr_Print();
       fprintf(stderr, "------------------------------------------------------\n");
       fprintf(stderr, "What does ImportError mean?\n");
       fprintf(stderr, "\n");
       fprintf(stderr, "It means that I couldn't load a python add-on for %s.theme\n", themeName.ascii());
       fprintf(stderr, "If this is a regular theme and doesn't use python\n");
       fprintf(stderr, "extensions, than nothing is wrong.\n");
       fprintf(stderr, "------------------------------------------------------\n");

     }

    //Clean up this thread's python interpreter reference
    PyThreadState_Swap(NULL);
    PyThreadState_Clear(myThreadState);
    PyThreadState_Delete(myThreadState);
    PyEval_ReleaseLock();
    //This thread's python refence is now gone.  A new reference must be created for each thread.


    widgetMask = 0;
    info = new NETWinInfo( qt_xdisplay(), winId(), qt_xrootwin(), NET::WMState );
    info->setDesktop( NETWinInfo::OnAllDesktops );
    info->setState( NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager,NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager );


    kWinModule = new KWinModule();
    connect( kWinModule,SIGNAL(currentDesktopChanged(int)), this,SLOT(currentDesktopChanged(int)) );

//     connect( &taskManager, SIGNAL(taskAdded(Task*)), this,SLOT(taskAdded(Task*)) );
//     connect( &taskManager, SIGNAL(taskRemoved(Task*)), this,SLOT(taskRemoved(Task*)) );


    kpop = new KPopupMenu( this );

    keditpop = new KPopupMenu ( this );

    keditpop -> insertItem(SmallIconSet("edit"),"Edit &theme",this,SLOT(editConfig()), CTRL+Key_E );
    keditpop -> insertItem(SmallIconSet("edit"),"Edit &script",this,SLOT(editScript()), ALT+Key_E, EDITSCRIPT);

    // Test if Theme Script exists
    QFileInfo scriptFile(themePath + "/" + themeName + ".py");
    keditpop -> setItemEnabled(EDITSCRIPT, scriptFile.exists());

    accColl = new KActionCollection( this );

    kpop->insertItem( SmallIconSet("reload"),"Update", this, SLOT(updateSensors()), Key_F5 );
    toggleLocked = new KToggleAction (  "Toggle &Locked position", CTRL+Key_L, this,
                                        SLOT( slotToggleLocked() ), accColl  , "Locked position" );
    accColl->insert(toggleLocked);
    toggleLocked->plug(kpop);

    kpop->insertSeparator();

    kpop->insertItem( SmallIconSet("fileopen"),"&Add new configuration", this, SLOT(startNewKaramba()), CTRL+Key_O );
    //    kpop->insertItem( SmallIconSet("edit"),"&Edit configuration",this,SLOT(editConfig()), CTRL+Key_E );
    kpop->insertItem("&Edit configuration", keditpop);
    kpop->insertItem( SmallIconSet("reload3"),"&Reload configuration",this,SLOT(reloadConfig()), CTRL+Key_R );
    kpop->insertItem( SmallIconSet("fileclose"),"&Close this configuration", this, SLOT(killWidget()), CTRL+Key_C );

    kpop->insertSeparator();

    kpop->insertItem( SmallIconSet("exit"),"&Quit", qApp, SLOT(quit()), CTRL+Key_Q );

       kpop->polish();

    foundKaramba = false;
    onTop = false;
    fixedPosition = false;


    themeFile = fn;

    meterList = new QObjectList();
    meterList->setAutoDelete( true );
    sensorList = new QObjectList();
    sensorList->setAutoDelete( true );
    menuList = new QObjectList();
    menuList->setAutoDelete( true );
    //clickList = new QObjectList();
    timeList = new QObjectList();
    imageList = new QObjectList();
    clickList = new list<Clickable*>;

    client = kapp->dcopClient();
    if (!client->isAttached())
        client->attach();
    appId = client->registerAs(qApp->name());


    this->setBackgroundMode( NoBackground);
    if( !onTop)
        this->lower();


    if( !parseConfig() )
    {
        setFixedSize(0,0);
        QTimer::singleShot( 100, this, SLOT(killWidget()) );

        qDebug("false");
    }
    else
    {
        kroot = new KarambaRootPixmap((QWidget*)this);
        kroot->start();


    }
    
    haveUpdated = 0;
    this->setMouseTracking(true);

}

karamba::~karamba()
{
    if(meterList != 0)
    {
        meterList->clear();
        delete meterList;
    }

    if( sensorList != 0 )
    {
        sensorList->clear();
        delete sensorList;
    }

    if( imageList != 0 )
    {
        imageList->clear();
        delete imageList;
    }
	
	if( clickList != 0 )
    {
        clickList->clear();
        delete clickList;
    }
	
	if( timeList != 0 )
    {
        timeList->clear();
        delete timeList;
    }

    if( toggleLocked != 0)
        delete toggleLocked;
    if( accColl != 0)
        delete accColl;

    if( keditpop != 0)
      delete keditpop;

    if( kpop != 0)
        delete kpop;

    if( widgetMask != 0)
      delete widgetMask;

    delete kWinModule;

    //Clean up Python references
    if (pythonThemeExtensionLoaded) {
      // get the global python thread lock
      PyEval_AcquireLock();
      
      // get a reference to the PyInterpreterState
      extern PyThreadState * mainThreadState;
      PyInterpreterState * mainInterpreterState = mainThreadState->interp;
      
      // create a thread state object for this thread
      PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
      PyThreadState_Swap(myThreadState);
      
      //Displose of current python module so we can reload in constructor.
      Py_DECREF(pModule);
      Py_DECREF(pName);
      
      //Clean up this thread's python interpreter reference
      PyThreadState_Swap(NULL);
      PyThreadState_Clear(myThreadState);
      PyThreadState_Delete(myThreadState);
      PyEval_ReleaseLock();
      //This thread's python refence is now gone.  A new reference must be created for each thread.
    }
}

bool karamba::parseConfig()
{
    QTimer *m_sysTimer = new QTimer(this);

    QFile file(themeFile);
    QString line;
    QString meter;
    int interval = 0;
    //    TextLabel *defaultTextLabel = 0;
    TextField *defaultTextField = 0;

    if ( file.open(IO_ReadOnly | IO_Translate) )
    {
        QTextStream t( &file );        // use a text stream
        QValueStack<QPoint> offsetStack;
        offsetStack.push(QPoint(0,0));

        int x=0;
        int y=0;
        int w=0;
        int h=0;
        while( ( line = t.readLine() ) !=0 )
        {
            QRegExp rx("^\\s*(\\S+)");
            rx.search(line);
            meter = rx.cap(1).upper();

            x = getInt("X",line)+offsetStack.top().x();
            y = getInt("Y",line)+offsetStack.top().y();
            w = getInt("W",line);
            h = getInt("H",line);

            //qDebug("x: " + QString::number(offsetStack.top().x()) + "  y: " + QString::number(offsetStack.top().y() ));


            if( meter == "KARAMBA" && !foundKaramba )
            {
	      //                qDebug("karamba");
                toggleLocked->setChecked( getBoolean( "LOCKED", line ) );
                slotToggleLocked();


                getInt( "X",line );
                getInt( "Y",line );

                x = ( x < 0 ) ? 0:x;
                y = ( y < 0 ) ? 0:y;
                move(x,y);


                if( w == 0 ||  h == 0)
                {
                    w = 300;
                    h = 300;
                }
                setFixedSize(w,h);

                if( getBoolean( "RIGHT", line ) )
                {
                    QDesktopWidget *d = QApplication::desktop();
                    x = d->width() - w;
                }
                else
                    if( getBoolean( "LEFT", line ) )
                    {
                        x = 0;
                    }

                if( getBoolean( "BOTTOM", line ) )
                {
                    QDesktopWidget *d = QApplication::desktop();
                    y = d->height() - h;
                }
                else
                    if( getBoolean( "TOP", line ) )
                    {
                        y = 0;
                    }

                pm = QPixmap(size());

                if( getBoolean("ONTOP", line ) )
                {
                    onTop = true;
                    KWin::setState( winId(), NET::StaysOnTop );
                }

                if( getBoolean("TOPBAR", line ) )
                {
                    move(x,0);
                    KWin::setStrut( winId(), 0, 0, h, 0 );
                    toggleLocked->setChecked( true );
                    slotToggleLocked();
                    toggleLocked->setEnabled(false);

                }

                if( getBoolean("BOTTOMBAR", line ) )
                {
                    int dh = QApplication::desktop()->height();
                    move( x, dh - h );
                    KWin::setStrut( winId(), 0, 0, 0, h );
                    toggleLocked->setChecked( true );
                    slotToggleLocked();
                    toggleLocked->setEnabled(false);


                }


		bool found = false;
                QString path = getString( "MASK", line, found );

                if( 1 )
                {		  

		  QFileInfo info(path);
		  if( info.isRelative())
                    path = themePath +"/" + path;

		  widgetMask = new QBitmap(path);
  
		  setMask(*widgetMask);
		  
                }

                interval = getInt("INTERVAL",line);
                interval = ( interval == 0 ) ? 5000 : interval;
                foundKaramba = true;
            }

            if( meter == "THEME" )
            {
                QString path = getString( "PATH", line );
                QFileInfo info(path);
                if( info.isRelative())
                    path = themePath +"/" + path;
		//                qDebug("path: "+path);
                (new karamba( path, false ))->show();
            }

            if( meter == "<GROUP>" )
            {
                int offsetX = offsetStack.top().x();
                int offsetY = offsetStack.top().y();
                offsetStack.push( QPoint( offsetX + getInt("X",line),
                                          offsetY + getInt("Y",line) ) );
            }

            if( meter == "</GROUP>" )
            {
                offsetStack.pop();
            }

            if( meter == "CLICKAREA" )
            {
                if( !hasMouseTracking() )
                    setMouseTracking(true);
                ClickArea *tmp = new ClickArea( x, y, w, h );
                tmp->setOnClick( getString( "ONCLICK", line ) );

                setSensor( line, (Meter*) tmp );
		//                clickList->append( tmp );
                clickList->push_back( tmp );
                if( getBoolean( "PREVIEW", line ) )
                    meterList->append( tmp );
            }

            // program sensor without a meter
            if(meter == "SENSOR=PROGRAM")
            {
                setSensor(line, 0 );
            }

            if(meter == "IMAGE")
            {
                QString file = getString("PATH",line);
                QString file_roll = getString("PATHROLL",line);
                int xon = getInt("XROLL",line);
                int yon = getInt("YROLL",line);
                QString tiptext = getString("TOOLTIP",line);
                xon = ( xon <= 0 ) ? x:xon;
                yon = ( yon <= 0 ) ? y:yon;

                ImageLabel *tmp = new ImageLabel(x,y,0,0);
                tmp->setThemePath( themePath );
                tmp->setValue( file );
				tmp->parseImages(file, file_roll, x,y, xon, yon);
				tmp->setKaramba(this);

				if (!tiptext.isEmpty())
					tmp->setTooltip(tiptext);

                //bool sensorFound = false;
                //getString( "SENSOR", line, sensorFound );
                //if( !sensorFound )

                setSensor(line, (Meter*) tmp );
                meterList->append (tmp );
                imageList->append (tmp );
            }

            if( meter == "DEFAULTFONT" )
            {
//                 int r,g,b;

//                 delete defaultTextLabel;
//                 defaultTextLabel = new TextLabel(x, y, w, h );

//                 get3Int( "COLOR", line, r, g, b );
//                 defaultTextLabel->setColor( r, g, b );
//                 get3Int( "BGCOLOR", line, r, g, b );
//                 defaultTextLabel->setBGColor( r, g, b );

//                 defaultTextLabel->setFont( getString( "FONT", line ) );
//                 defaultTextLabel->setFontSize( getInt( "FONTSIZE", line ) );
//                 defaultTextLabel->setAlignment( getString( "ALIGN", line ) );
//                 defaultTextLabel->setFixedPitch( getBoolean( "FIXEDPITCH", line ) );
//                 defaultTextLabel->setShadow( getInt( "SHADOW", line ) );
                int r,g,b;

                delete defaultTextField;
                defaultTextField = new TextField( );

                get3Int( "COLOR", line, r, g, b );
                defaultTextField->setColor( r, g, b );
                get3Int( "BGCOLOR", line, r, g, b );
                defaultTextField->setBGColor( r, g, b );

                defaultTextField->setFont( getString( "FONT", line ) );
                defaultTextField->setFontSize( getInt( "FONTSIZE", line ) );
                defaultTextField->setAlignment( getString( "ALIGN", line ) );
                defaultTextField->setFixedPitch( getBoolean( "FIXEDPITCH", line ) );
                defaultTextField->setShadow( getInt( "SHADOW", line ) );

            }


            if(meter == "TEXT" || meter == "CLICKMAP")
            {
//                 int r,g,b;

//                 TextLabel *tmp = new TextLabel(x, y, w, h );
//                 bool paramFound = false;

//                 get3Int( "COLOR", line, r, g, b, paramFound );
//                 if ( paramFound || !defaultTextLabel )
//                     tmp->setColor ( r, g, b );
//                 else
//                     if ( defaultTextLabel )
//                     {
//                         defaultTextLabel->getColor().rgb ( &r, &g, &b );
//                         tmp->setColor( r, g, b );
//                     }

//                 get3Int( "BGCOLOR", line, r, g, b, paramFound );
//                 if ( paramFound  || !defaultTextLabel )
//                     tmp->setBGColor( r, g, b );
//                 else
//                     if ( defaultTextLabel )
//                     {
//                         defaultTextLabel->getBGColor().rgb ( &r, &g, &b );
//                         tmp->setBGColor( r, g, b );
//                     }

//                 QString dFont = getString("FONT", line, paramFound);
//                 if( paramFound || !defaultTextLabel )
//                     tmp->setFont( dFont );
//                 else
//                     if( defaultTextLabel )
//                         tmp->setFont( defaultTextLabel->getFont() );


//                 int dFontSize = getInt( "FONTSIZE", line, paramFound );
//                 if( paramFound || !defaultTextLabel )
//                     tmp->setFontSize( dFontSize );
//                 else
//                     if( defaultTextLabel )
//                         tmp->setFontSize( defaultTextLabel->getFontSize() );

//                 QString dAlign = getString( "ALIGN", line, paramFound );
//                 if( paramFound || !defaultTextLabel )
//                     tmp->setAlignment( dAlign );
//                 else
//                     if( defaultTextLabel )
//                         tmp->setAlignment( defaultTextLabel->getAlignment() );

//                 bool dFp = getBoolean( "FIXEDPITCH", line, paramFound );
//                 if( paramFound || !defaultTextLabel )
//                     tmp->setFixedPitch( dFp );
//                 else
//                     if( defaultTextLabel )
//                         tmp->setFixedPitch( defaultTextLabel->getFixedPitch() );

//                 int dShadow = getInt( "SHADOW", line, paramFound );
//                 if( paramFound || !defaultTextLabel )
//                     tmp->setShadow( dShadow );
//                 else
//                     if( defaultTextLabel )
//                         tmp->setShadow( defaultTextLabel->getShadow() );

//                 tmp->setValue( getString( "VALUE", line ) );

//                 setSensor( line, (Meter*)tmp );
//                 meterList->append ( tmp );
                int r,g,b;

                TextField* tmpText = new TextField();
                bool paramFound = false;

                get3Int( "COLOR", line, r, g, b, paramFound );
                if ( paramFound || !defaultTextField ) {
                    tmpText->setColor ( r, g, b );
		}
                else
                    if ( defaultTextField )
                    {
                        defaultTextField->getColor().rgb ( &r, &g, &b );
                        tmpText->setColor( r, g, b );
                    }

                get3Int( "BGCOLOR", line, r, g, b, paramFound );
                if ( paramFound  || !defaultTextField )
                    tmpText->setBGColor( r, g, b );
                else
                    if ( defaultTextField )
                    {
                        defaultTextField->getBGColor().rgb ( &r, &g, &b );
                        tmpText->setBGColor( r, g, b );
                    }

                QString dFont = getString("FONT", line, paramFound);
                if( paramFound || !defaultTextField )
                    tmpText->setFont( dFont );
                else
                    if( defaultTextField )
                        tmpText->setFont( defaultTextField->getFont() );


                int dFontSize = getInt( "FONTSIZE", line, paramFound );
                if( paramFound || !defaultTextField )
                    tmpText->setFontSize( dFontSize );
                else
                    if( defaultTextField )
                        tmpText->setFontSize( defaultTextField->getFontSize() );

                QString dAlign = getString( "ALIGN", line, paramFound );
                if( paramFound || !defaultTextField )
                    tmpText->setAlignment( dAlign );
                else
                    if( defaultTextField )
                        tmpText->setAlignment( defaultTextField->getAlignment() );

                bool dFp = getBoolean( "FIXEDPITCH", line, paramFound );
                if( paramFound || !defaultTextField )
                    tmpText->setFixedPitch( dFp );
                else
                    if( defaultTextField )
                        tmpText->setFixedPitch( defaultTextField->getFixedPitch() );

                int dShadow = getInt( "SHADOW", line, paramFound );
                if( paramFound || !defaultTextField )
                    tmpText->setShadow( dShadow );
                else
                    if( defaultTextField )
                        tmpText->setShadow( defaultTextField->getShadow() );

                // ////////////////////////////////////////////////////
                // Now handle the specifics
                if( meter == "TEXT" )
                {
                    
                    TextLabel *tmp = new TextLabel( x, y, w, h );
                    tmp->setTextProps( tmpText );
                    tmp->setValue( getString( "VALUE", line ) );

                    setSensor( line, (Meter*)tmp );
                    meterList->append ( tmp );
                }

                if(meter == "CLICKMAP")
                {
                    if( !hasMouseTracking() )
                        setMouseTracking(true);
                    ClickMap *tmp = new ClickMap( x, y, w, h );
                    tmp->setTextProps( tmpText );

                    setSensor( line, (Meter*) tmp );
                    // set all params
                    clickList->push_back( tmp );
                    meterList->append( tmp );

                }

            }

            if(meter == "BAR")
            {
                Bar *tmp =
                    new Bar( x, y, w, h );
                tmp->setThemePath( themePath );
                tmp->setImage( getString( "PATH", line ) );
                tmp->setVertical( getBoolean( "VERTICAL", line ) );
                bool maxFound = false;
                int max = getInt( "MAX", line, maxFound );
                if( maxFound )
                    tmp->setMax( max );
                bool minFound = false;
                int min = getInt( "MIN", line, minFound );
                if( minFound )
                    tmp->setMin( min );
                tmp->setValue( getInt( "VALUE", line ) );
                setSensor( line, (Meter*)tmp );
                meterList->append ( tmp );
            }

            if(meter == "GRAPH")
            {
                int r, g, b;
                int points = getInt("POINTS",line);
                get3Int("COLOR", line, r, g, b);

                Graph *tmp = new Graph( x, y, w, h, points );
                bool maxFound = false;
                int max = getInt( "MAX", line, maxFound );
                if( maxFound )
                    tmp->setMax( max );
                bool minFound = false;
                int min = getInt( "MIN", line, minFound );
                if( minFound )
                    tmp->setMin( min );

                tmp->setColor( r , g, b );

                setSensor( line, (Graph*)tmp );
                meterList->append ( tmp );
            }
        }
    }
    if( !foundKaramba )
    {
        //  interval = initKaramba( "", 0, 0, 0, 0 );
        //   this->close(true);
        //delete this;
        return false;
    }
    else
    {
        connect(m_sysTimer, SIGNAL(timeout()), SLOT(step()));

        interval = interval == 0 ? 1000 : interval;
        m_sysTimer->start(interval);
        QTimer::singleShot( 1000, this, SLOT(step()) );
        QTimer::singleShot( 5000, this, SLOT(step()) );
        QTimer::singleShot( 10000, this, SLOT(step()) );

        if( !onTop )
            lowerTimer.start();
        //update();
        //QTimer::singleShot( 0, this, SLOT(update()) );
        return true;
    }
}

void karamba::startNewKaramba()
{
    QStringList fileNames;
    fileNames = KFileDialog::getOpenFileNames(QString::null, "*.theme", 0, "Open Themes");
    for ( QStringList::Iterator it = fileNames.begin(); it != fileNames.end(); ++it )
    {
        QFileInfo file( *it );
        if( file.exists() )
        {
            (new karamba( *it, false ))->show();
        }
    }
}

void karamba::popupNotify(int item)
{
  
}

void karamba::reloadConfig()
{

    QFileInfo file( themeFile );
    //qDebug(themeFile);
    if( file.exists() )
    {
        (new karamba( themeFile, true ))->show();
    }
    killWidget();
}

void karamba::killWidget() {


  close();
}


// void karamba::taskAdded(Task* t) { 

//     if (pythonThemeExtensionLoaded && haveUpdated) {

//       // get the global lock
//       PyEval_AcquireLock();
//       // get a reference to the PyInterpreterState
      
//       //Python thread references
//       extern PyThreadState * mainThreadState;
      
//       PyInterpreterState * mainInterpreterState = mainThreadState->interp;
//       // create a thread state object for this thread
//       PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
//       PyThreadState_Swap(myThreadState);
            
//       pFunc = PyDict_GetItemString(pDict, "widgetTaskAdded");
      
//       if (pFunc && PyCallable_Check(pFunc)) {
// 	pArgs = PyTuple_New(2);
	
// 	pValue = PyInt_FromLong((int)this);
// 	PyTuple_SetItem(pArgs, 0, pValue);
	
// 	pValue = PyInt_FromLong((int)t);
// 	PyTuple_SetItem(pArgs, 1, pValue);
	
	
	
// 	pValue = PyObject_CallObject(pFunc, pArgs);
// 	if (pValue != NULL) {
// 	  Py_DECREF(pValue);
// 	}
// 	else {
// 	  PyErr_Print();
// 	  fprintf(stderr,"Call to widgetTaskAdded failed\n");
// 	}
// 	Py_DECREF(pArgs);
//       }
//       else {
// 	PyErr_Print();
// 	fprintf(stderr, "Cannot find python widgetClicked callback function\n");
//       }
      
      
//       // swap my thread state out of the interpreter
//       PyThreadState_Swap(NULL);
//       // clear out any cruft from thread state object
//       PyThreadState_Clear(myThreadState);
//       // delete my thread state object
//       PyThreadState_Delete(myThreadState);
//       // release the lock
//       PyEval_ReleaseLock();
//     }

// }

// void karamba::taskRemoved(Task* t) {
//     if (pythonThemeExtensionLoaded && haveUpdated) {

//       // get the global lock
//       PyEval_AcquireLock();
//       // get a reference to the PyInterpreterState
      
//       //Python thread references
//       extern PyThreadState * mainThreadState;
      
//       PyInterpreterState * mainInterpreterState = mainThreadState->interp;
//       // create a thread state object for this thread
//       PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
//       PyThreadState_Swap(myThreadState);
            
//       pFunc = PyDict_GetItemString(pDict, "widgetTaskRemoved");
      
//       if (pFunc && PyCallable_Check(pFunc)) {
// 	pArgs = PyTuple_New(2);
	
// 	pValue = PyInt_FromLong((int)this);
// 	PyTuple_SetItem(pArgs, 0, pValue);
	
// 	pValue = PyInt_FromLong((int)t);
// 	PyTuple_SetItem(pArgs, 1, pValue);
	
	
	
// 	pValue = PyObject_CallObject(pFunc, pArgs);
// 	if (pValue != NULL) {
// 	  Py_DECREF(pValue);
// 	}
// 	else {
// 	  PyErr_Print();
// 	  fprintf(stderr,"Call to widgetTaskRemoved failed\n");
// 	}
// 	Py_DECREF(pArgs);
//       }
//       else {
// 	PyErr_Print();
// 	fprintf(stderr, "Cannot find python widgetClicked callback function\n");
//       }
      
      
//       // swap my thread state out of the interpreter
//       PyThreadState_Swap(NULL);
//       // clear out any cruft from thread state object
//       PyThreadState_Clear(myThreadState);
//       // delete my thread state object
//       PyThreadState_Delete(myThreadState);
//       // release the lock
//       PyEval_ReleaseLock();
//     }

// }

void karamba::editConfig()
{
    QFileInfo fileInfo( themeFile );
    QString path;

    if( fileInfo.isRelative() )
    {
        path = themePath + "/" + themeFile;
    }
    else
    {
        path = themeFile;
    }
    //    qDebug("krun:"+path);
    KRun::runURL( KURL( path ), "text/plain" );

}

void karamba::editScript()
{
    QFileInfo fileInfo( themeFile );
    QString path;

    if( fileInfo.isRelative() )
    {
        path = themePath + "/" + themeName + ".py";
    }
    else
    {
        path = QFileInfo(themeFile).dirPath() + "/" + themeName + ".py";
    }
    //    qDebug("krun:"+path);
    KRun::runURL( KURL( path ), "text/plain" );

}


int karamba::getInt(QString w, QString &line )
{
    bool b;
    return getInt( w, line, b );
}

int karamba::getInt(QString w, QString &line, bool &found )
{
    QRegExp rx( "\\W+" + w +"=([-]?\\d+)", false );
    found = (rx.search(line)==-1)?false:true;
    return rx.cap(1).toInt();
}

void karamba::get3Int(QString w, QString &line, int &a, int &b, int &c )
{
    bool bo;
    get3Int ( w, line, a, b, c, bo );
}

void karamba::get3Int(QString w, QString &line, int &a, int &b, int &c, bool &found )
{
    QRegExp rx( "\\W+" + w + "=([-]?\\d+),([-]?\\d+),([-]?\\d+)", false );
    found = (rx.search(line)==-1)?false:true;
    a = rx.cap(1).toInt();
    b = rx.cap(2).toInt();
    c = rx.cap(3).toInt();
}

QString karamba::getString(QString w, QString &line)
{
    bool b;
    return getString ( w, line, b );
}

QString karamba::getString(QString w, QString &line, bool &found)
{
    QRegExp rx( "\\W+" + w + "=\"([^\"]*)\"", false );
    found = (rx.search(line)==-1)?false:true;
    if (rx.cap(1).isEmpty())
    {
        rx = QRegExp(w + "=(\\S+)", false);
        rx.search(line);
        return rx.cap(1);
    }
    else
    {
        return rx.cap(1);
    }

}

bool karamba::getBoolean( QString w, QString &line )
{
    bool b;
    return getBoolean ( w, line, b );
}

bool karamba::getBoolean( QString w, QString &line, bool &found )
{
    return  ( getString( w, line, found ).lower() == "true")?true:false;
}

void karamba::setSensor(QString &line, Meter* meter)
{
    Sensor* sensor = 0;

    QString sens = getString("SENSOR",line).upper();

    if( sens == "CPU" )
    {
        QString cpuNbr = getString( "CPU", line );
        sensor = sensorMap["CPU"+cpuNbr];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?1000:interval;
            sensor = ( sensorMap["CPU"+cpuNbr] = new CPUSensor( cpuNbr, interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sp->addParam("DECIMALS",getString("DECIMALS",line));

        sensor->addMeter(sp);
        sensor->setMaxValue(sp);

    }

    if( sens == "MEMORY" )
    {
        sensor = sensorMap["MEMORY"];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?3000:interval;
            sensor = ( sensorMap["MEMORY"] = new MemSensor( interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));

        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }


    if( sens == "DISK" )
    {
        sensor = sensorMap["DISK"];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?5000:interval;
            sensor = ( sensorMap["DISK"] = new DiskSensor( interval ) );
            sensorList->append( sensor );
        }
        // meter->setMax( ((DiskSensor*)sensor)->getTotalSpace(mntPt)/1024 );
        SensorParams *sp = new SensorParams(meter);
        QString mntPt = getString("MOUNTPOINT",line);
        if( mntPt == ""  )
            mntPt = "/";

        sp->addParam("MOUNTPOINT",mntPt);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }

    if( sens == "NETWORK")
    {
        int interval = getInt("INTERVAL",line );
        interval = (interval == 0)?2000:interval;
        QString device = getString("DEVICE", line );
        sensor = sensorMap["NETWORK"+device];
        if (sensor == 0)
        {
            sensor = ( sensorMap["NETWORK"+device] = new NetworkSensor( device, interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam( "FORMAT", getString( "FORMAT", line ) );
        sp->addParam( "DECIMALS", getString( "DECIMALS", line ) );
        sensor->addMeter(sp);
    }

    if( sens == "UPTIME" )
    {
        sensor = sensorMap["UPTIME"];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?60000:interval;
            sensor = ( sensorMap["UPTIME"] = new UptimeSensor( interval ));
            sensorList->append( sensor );

        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam( "FORMAT", getString( "FORMAT", line ) );
        sensor->addMeter(sp);
    }

    if( sens == "SENSOR" )
    {
        sensor = sensorMap["SENSOR"];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?30000:interval;
            sensor = ( sensorMap["SENSOR"] = new SensorSensor( interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sp->addParam("TYPE",getString("TYPE",line));
        sensor->addMeter(sp);
    }


    if( sens == "TEXTFILE" )
    {
        QString path = getString( "PATH", line );
        bool rdf = getBoolean( "RDF", line );
        sensor = sensorMap["TEXTFILE"+path];
        if (sensor == 0)
        {
            int interval = getInt( "INTERVAL", line );
            interval = ( interval == 0 )?60000:interval;
            QString encoding = getString( "ENCODING", line );

            sensor = ( sensorMap["TEXTFILE"+path] =
                           new TextFileSensor( path, rdf, interval, encoding ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("LINE",QString::number(getInt("LINE",line)));
        sensor->addMeter(sp);
    }


    if( sens == "TIME")
    {
        sensor = sensorMap["DATE"];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?60000:interval;
            sensor = ( sensorMap["DATE"] = new DateSensor( interval ) );
            sensorList->append( sensor );
			timeList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sp->addParam("CALWIDTH",getString("CALWIDTH",line));
        sp->addParam("CALHEIGHT",getString("CALHEIGHT",line));
        sensor->addMeter(sp);
    }

#ifdef HAVE_XMMS

    if( sens == "XMMS" )
    {
        sensor = sensorMap["XMMS"];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?1000:interval;
            QString encoding = getString( "ENCODING", line );

            sensor = ( sensorMap["XMMS"] = new XMMSSensor( interval, encoding ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }
#endif // HAVE_XMMS


    if( sens == "NOATUN" )
    {
        sensor = sensorMap["NOATUN"];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?1000:interval;
            sensor = ( sensorMap["NOATUN"] = new NoatunSensor( interval, client ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }

    if( sens == "PROGRAM")
    {
        QString progName = getString("PROGRAM",line);
        sensor = sensorMap["PROGRAM"+progName];
        if (sensor == 0)
        {
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?3600000:interval;
            QString encoding = getString( "ENCODING", line );

            sensor = (sensorMap["PROGRAM"+progName] =
                          new ProgramSensor( progName, interval, encoding ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam( "LINE", QString::number( getInt( "LINE", line ) ) );
        sp->addParam( "THEMAPATH", themePath );
        sensor->addMeter(sp);
    }

    if( sens == "RSS" )
    {
        QString source = getString( "SOURCE", line );
        QString format = getString( "FORMAT", line );

        sensor = sensorMap["RSS"+source];
        if (sensor == 0)
        {
            int interval = getInt( "INTERVAL", line );
            interval = ( interval == 0 )?60000:interval;
            QString encoding = getString( "ENCODING", line );

            sensor = ( sensorMap["RSS"+source] =
                           new RssSensor( source, interval, format, encoding ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("SOURCE",getString("SOURCE",line));
        sensor->addMeter(sp);
    }

    if (sensor != 0)
    {
        QTimer::singleShot( 0, sensor, SLOT(update()) );
        sensor->start();
    }
}


void karamba::passClick( QMouseEvent *e )
{
//     QObjectListIt it( *clickList ); // iterate over meters
//     while ( it != 0 )
//     {
//         (( ClickArea* ) *it)->click( e );
//         ++it;
//     }

	QObjectListIt it2( *timeList ); // iterate over meters
    while ( it2 != 0 )
    {
        (( DateSensor* ) *it2)->toggleCalendar( e );
        ++it2;
    }
    for(list<Clickable*>::iterator itr = clickList->begin(); itr != clickList->end(); ++itr)
    {
        ((Clickable*)(*itr))->click(e);
    }

    //Everything below is to call the python callback function

    if (pythonThemeExtensionLoaded && haveUpdated) {

      // get the global lock
      PyEval_AcquireLock();
      // get a reference to the PyInterpreterState
      
      //Python thread references
      extern PyThreadState * mainThreadState;
      
      PyInterpreterState * mainInterpreterState = mainThreadState->interp;
      // create a thread state object for this thread
      PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
      PyThreadState_Swap(myThreadState);

      pFunc = PyDict_GetItemString(pDict, "widgetClicked");
      
      if (pFunc && PyCallable_Check(pFunc)) {
	pArgs = PyTuple_New(4);
	
	pValue = PyInt_FromLong((int)this);
	PyTuple_SetItem(pArgs, 0, pValue);
	
	pValue = PyInt_FromLong(e->x());
	PyTuple_SetItem(pArgs, 1, pValue);
	
	pValue = PyInt_FromLong(e->y());
	PyTuple_SetItem(pArgs, 2, pValue);
	
	int button = 0;
	
	if( e->button() == Qt::LeftButton )
	  button = 1;
	else if( e->button() == Qt::MidButton )
	  button = 2;
	else if( e->button() == Qt::RightButton )
	  button = 3;
	
	pValue = PyInt_FromLong(button);
	PyTuple_SetItem(pArgs, 3, pValue);
	
	
	pValue = PyObject_CallObject(pFunc, pArgs);
	if (pValue != NULL) {
	  Py_DECREF(pValue);
	}
	else {
	  PyErr_Print();
	  fprintf(stderr,"Call to widgetClicked failed\n");
	}
	Py_DECREF(pArgs);
      }
      else {
	PyErr_Print();
	fprintf(stderr, "Cannot find python widgetClicked callback function\n");
      }
      
      
      // swap my thread state out of the interpreter
      PyThreadState_Swap(NULL);
      // clear out any cruft from thread state object
      PyThreadState_Clear(myThreadState);
      // delete my thread state object
      PyThreadState_Delete(myThreadState);
      // release the lock
      PyEval_ReleaseLock();
    }
}


void karamba::passWheelClick( QWheelEvent *e )
{
    //Everything below is to call the python callback function

    if (pythonThemeExtensionLoaded && haveUpdated) {

      // get the global lock
      PyEval_AcquireLock();
      // get a reference to the PyInterpreterState
      
      //Python thread references
      extern PyThreadState * mainThreadState;
      
      PyInterpreterState * mainInterpreterState = mainThreadState->interp;
      // create a thread state object for this thread
      PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
      PyThreadState_Swap(myThreadState);
      
      
      //    PyRun_SimpleString("iconbar.yo()");
      
      pFunc = PyDict_GetItemString(pDict, "widgetClicked");
      
      if (pFunc && PyCallable_Check(pFunc)) {
	pArgs = PyTuple_New(4);
	
	pValue = PyInt_FromLong((int)this);
	PyTuple_SetItem(pArgs, 0, pValue);
	
	pValue = PyInt_FromLong(e->x());
	PyTuple_SetItem(pArgs, 1, pValue);
	
	pValue = PyInt_FromLong(e->y());
	PyTuple_SetItem(pArgs, 2, pValue);
	
	int button = 0;
	
	if( e->delta() > 0 )
	  
	  button = 4;
	else
	  button = 5;
	
	
	
	pValue = PyInt_FromLong(button);
	PyTuple_SetItem(pArgs, 3, pValue);
	
	
	pValue = PyObject_CallObject(pFunc, pArgs);
	if (pValue != NULL) {
	  Py_DECREF(pValue);
	}
	else {
	  PyErr_Print();
	  fprintf(stderr,"Call to widgetClicked failed\n");
	}
	Py_DECREF(pArgs);
      }
      else {
	PyErr_Print();
	fprintf(stderr, "Cannot find python widgetClicked callback function\n");
      }
      
      
      // swap my thread state out of the interpreter
      PyThreadState_Swap(NULL);
      // clear out any cruft from thread state object
      PyThreadState_Clear(myThreadState);
      // delete my thread state object
      PyThreadState_Delete(myThreadState);
      // release the lock
      PyEval_ReleaseLock();
    }
}

void karamba::mousePressEvent( QMouseEvent *e )
{

    if( e->button() == RightButton )
    {

        kpop->exec(QCursor::pos());
    }
    else
    {
        clickPos = e->pos();
        if( locked )
            passClick( e );
        if( !onTop )
            lower();
    }
}

void karamba::wheelEvent( QWheelEvent *e )
{  
  passWheelClick( e );
}

void karamba::mouseReleaseEvent( QMouseEvent *e )
{
    /*if( ks != 0)
    {
        ks->loadFromShared("DESKTOP1",
                           QRect(QPoint(x(),y()), QPoint(x()+width()-1,y()+height()-1)));
    }
    */
    clickPos = e->pos();
    //    qDebug("Click   X="+QString::number( clickPos.x() )+" Y="+QString::number( clickPos .y() ) );
    //    qDebug("Window  X="+QString::number( ( e->globalPos() - clickPos ).x())+" Y="+QString::number( ( e->globalPos() - clickPos ).y()));

    //update();
    step();
}

void karamba::mouseDoubleClickEvent( QMouseEvent *e )
{
    if( !locked )
    {
        passClick( e );
    }
}



void karamba::mouseMoveEvent( QMouseEvent *e )
{


    if( e->state() !=  0 && e->state() < 16 && !locked )
    {
        move( e->globalPos() - clickPos );
    }
    else
    {
        // Change cursor over ClickArea
      //        QObjectListIt it( *clickList ); // iterate over meters
        list<Clickable*>::iterator it = clickList->begin(); // iterate over meters
        bool insideArea = false;
	//      while ( it != 0 && !insideArea )
while ( it != clickList->end() && !insideArea )

        {
	  //            insideArea = (((ClickArea*) *it)->getRectangle()).contains(e->pos());
	    insideArea = (((Clickable*) *it)->getRectangle()).contains(e->pos());
 
            ++it;
        }
        if(insideArea)
        {
            if( cursor().shape() != PointingHandCursor )
                setCursor( PointingHandCursor );
        }
        else
        {
            if( cursor().shape() != ArrowCursor )
                setCursor( ArrowCursor );
        }

		QObjectListIt image_it( *imageList);	// iterate over image sensors
		while ( image_it != 0 )
		{
			((ImageLabel*) *image_it)->rolloverImage(e);
			++image_it;
		}
    }


    if (pythonThemeExtensionLoaded && haveUpdated) {

      // get the global lock
      PyEval_AcquireLock();
      // get a reference to the PyInterpreterState
      
      //Python thread references
      extern PyThreadState * mainThreadState;
      
      PyInterpreterState * mainInterpreterState = mainThreadState->interp;
      // create a thread state object for this thread
      PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
      PyThreadState_Swap(myThreadState);
      
      pFunc = PyDict_GetItemString(pDict, "widgetMouseMoved");
      
      if (pFunc && PyCallable_Check(pFunc)) {
	pArgs = PyTuple_New(4);

	pValue = PyInt_FromLong((int)this);
	PyTuple_SetItem(pArgs, 0, pValue);
	
	pValue = PyInt_FromLong(e->x());
	PyTuple_SetItem(pArgs, 1, pValue);
	
	pValue = PyInt_FromLong(e->y());
	PyTuple_SetItem(pArgs, 2, pValue);
	
	int button = 0;
	
	if( e->button() == Qt::LeftButton )
	  button = 1;
	else if( e->button() == Qt::MidButton )
	  button = 2;
	else if( e->button() == Qt::RightButton )
	  button = 3;
	
	pValue = PyInt_FromLong(button);
	PyTuple_SetItem(pArgs, 3, pValue);
	
	
	pValue = PyObject_CallObject(pFunc, pArgs);
	if (pValue != NULL) {
	  Py_DECREF(pValue);
	}
	else {
	  PyErr_Print();
	  fprintf(stderr,"Call to widgetMouseMoved failed\n");
	}
	Py_DECREF(pArgs);
      }
      else {
	PyErr_Print();
	fprintf(stderr, "Cannot find python widgetMouseMoved callback function\n");
      }
      

      // swap my thread state out of the interpreter
      PyThreadState_Swap(NULL);
      // clear out any cruft from thread state object
      PyThreadState_Clear(myThreadState);
      // delete my thread state object
      PyThreadState_Delete(myThreadState);
      // release the lock
      PyEval_ReleaseLock();
    }
}

void karamba::closeEvent ( QCloseEvent *  )
{
  //Tell python scripts they are getting killed so they can react.
//     if (pythonThemeExtensionLoaded) {

//       // get the global lock
//       PyEval_AcquireLock();
//       // get a reference to the PyInterpreterState
      
//       //Python thread references
//       extern PyThreadState * mainThreadState;
      
//       PyInterpreterState * mainInterpreterState = mainThreadState->interp;
//       // create a thread state object for this thread
//       PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
//       PyThreadState_Swap(myThreadState);
            
//       pDict = PyModule_GetDict(pModule);
//       pFunc = PyDict_GetItemString(pDict, "widgetKilled");
      
//       if (pFunc && PyCallable_Check(pFunc)) {
// 	pArgs = PyTuple_New(0);
	
// 	pValue = PyObject_CallObject(pFunc, pArgs);
// 	if (pValue != NULL) {
// 	  Py_DECREF(pValue);
// 	}
// 	else {
// 	  PyErr_Print();
// 	  fprintf(stderr,"Call to widgetKilled failed\n");
// 	}
// 	Py_DECREF(pArgs);
//       }
//       else {
// 	PyErr_Print();
// 	fprintf(stderr, "Cannot find python widgetKilled callback function\n");
//       }
      
      
//       // swap my thread state out of the interpreter
//       PyThreadState_Swap(NULL);
//       // clear out any cruft from thread state object
//       PyThreadState_Clear(myThreadState);
//       // delete my thread state object
//       PyThreadState_Delete(myThreadState);
//       // release the lock
//       PyEval_ReleaseLock();
//     }
  this->close(true);
  delete this;
}

void karamba::paintEvent ( QPaintEvent *e)
{
    if( !onTop )
        if( lowerTimer.elapsed() > 100 )
        {
            this->lower();
            lowerTimer.restart();
        }
    QRect rect = e->rect();
    bitBlt(this,rect.topLeft(),&pm,rect,Qt::CopyROP);
}

void karamba::updateSensors()
{
    QObjectListIt it( *sensorList ); // iterate over meters
    while ( it != 0 )
    {
        ((Sensor*) *it)->update();
        ++it;
    }
    QTimer::singleShot( 500, this, SLOT(step()) );
}

void karamba::updateBackground(KSharedPixmap* kpm)
{
    background = QPixmap(*kpm);
    step();


}

void karamba::currentDesktopChanged( int )
{
    kroot->repaint( true );
}


void karamba::externalStep()
{
    pm = QPixmap(size());

    QObjectListIt it( *meterList ); // iterate over meters
    p.begin(&pm);
    bitBlt(&pm,0,0,&background,0,Qt::CopyROP);

    while ( it != 0 )
    {
        ((Meter*) *it)->mUpdate(&p);
        ++it;
    }
    p.end();
    repaint();
    //    update();
    //    kroot->repaint(true);
}


void karamba::step()
{
    pm = QPixmap(size());

    QObjectListIt it( *meterList ); // iterate over meters
    p.begin(&pm);
    bitBlt(&pm,0,0,&background,0,Qt::CopyROP);

    while ( it != 0 )
    {
        ((Meter*) *it)->mUpdate(&p);
        ++it;
    }
    p.end();
    update();


    if (pythonThemeExtensionLoaded) {
      
      // get the global lock
      PyEval_AcquireLock();
      // get a reference to the PyInterpreterState
      
      //Python thread references
      extern PyThreadState * mainThreadState;
      
      PyInterpreterState * mainInterpreterState = mainThreadState->interp;
      // create a thread state object for this thread
      PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
      PyThreadState_Swap(myThreadState);

      if (haveUpdated == 0) {
	pFunc = PyDict_GetItemString(pDict, "initWidget");
	haveUpdated = 1;
      }
      else 
	pFunc = PyDict_GetItemString(pDict, "widgetUpdated");
        
      
      if (pFunc && PyCallable_Check(pFunc)) {
	pArgs = PyTuple_New(1);

	  pValue = PyInt_FromLong((int)this);
	  PyTuple_SetItem(pArgs, 0, pValue);
	
	pValue = PyObject_CallObject(pFunc, pArgs);
	if (pValue != NULL) {
	  Py_DECREF(pValue);
	}
	else {
	  PyErr_Print();
	  fprintf(stderr,"Call to widgetUpdated failed\n");
	}
	Py_DECREF(pArgs);
      }
      else {
	PyErr_Print();
	fprintf(stderr, "Cannot find python widgetMouseMoved callback function\n");
      }
      

      // swap my thread state out of the interpreter
      PyThreadState_Swap(NULL);
      // clear out any cruft from thread state object
      PyThreadState_Clear(myThreadState);
      // delete my thread state object
      PyThreadState_Delete(myThreadState);
      // release the lock
      PyEval_ReleaseLock();

    }
}

void karamba::slotToggleLocked()
{
  //    qDebug("toggleLocked");
    locked = toggleLocked->isChecked();
    if( locked )
    {
        toggleLocked->setIconSet(SmallIconSet("encrypted"));
    }
    else
    {
        toggleLocked->setIconSet(SmallIconSet("lockoverlay"));
    }

}

void karamba::passMenuItemClicked(int id)
{
    //Everything below is to call the python callback function

    if (pythonThemeExtensionLoaded) {

      // get the global lock
      PyEval_AcquireLock();
      // get a reference to the PyInterpreterState

      //Python thread references
      extern PyThreadState * mainThreadState;

      PyInterpreterState * mainInterpreterState = mainThreadState->interp;
      // create a thread state object for this thread
      PyThreadState * myThreadState = PyThreadState_New(mainInterpreterState);
      PyThreadState_Swap(myThreadState);

      pFunc = PyDict_GetItemString(pDict, "menuItemClicked");

      if (pFunc && PyCallable_Check(pFunc)) {
	pArgs = PyTuple_New(3);

	pValue = PyInt_FromLong((int)this);
	PyTuple_SetItem(pArgs, 0, pValue);

        int menu = 0;
	for(int i = 0; i < menuList->count(); i++){
          KPopupMenu* tmp;
	  if(i==0){
	    tmp = (KPopupMenu*) menuList->first();
	  } else {
	    tmp = (KPopupMenu*) menuList->next();
	  }
	  if(tmp != 0){
	    if(tmp->isItemVisible(id)){
	      menu = (int) tmp;
	      break;
	    }
	  }
	}

	pValue = PyInt_FromLong(menu);
	PyTuple_SetItem(pArgs, 1, pValue);

	pValue = PyInt_FromLong(id);
	PyTuple_SetItem(pArgs, 2, pValue);


	pValue = PyObject_CallObject(pFunc, pArgs);
	if (pValue != NULL) {
	  Py_DECREF(pValue);
	}
	else {
	  PyErr_Print();
	  fprintf(stderr,"Call to menuItemClicked failed\n");
	}
	Py_DECREF(pArgs);
      }
      else {
	PyErr_Print();
	fprintf(stderr, "Cannot find python menuItemClicked callback function\n");
      }


      // swap my thread state out of the interpreter
      PyThreadState_Swap(NULL);
      // clear out any cruft from thread state object
      PyThreadState_Clear(myThreadState);
      // delete my thread state object
      PyThreadState_Delete(myThreadState);
      // release the lock
      PyEval_ReleaseLock();
    }
}


#include "karamba.moc"
